@@ -1,8 +1,8 @@
 import asyncio
-import functools
 import os
 import pathlib
-import platform
+import ssl
+import sys
 from re import match as match_regex
 from unittest import mock
 from uuid import uuid4
@@ -13,35 +13,10 @@ from yarl import URL
 
 import aiohttp
 from aiohttp import web
-from aiohttp.client_exceptions import ClientConnectionError, ClientProxyConnectionError
-from aiohttp.helpers import IS_MACOS, IS_WINDOWS, PY_310
+from aiohttp.client_exceptions import ClientConnectionError
+from aiohttp.helpers import IS_MACOS, IS_WINDOWS
 
-pytestmark = [
-    pytest.mark.filterwarnings(
-        "ignore:unclosed <socket.socket fd=.*:ResourceWarning",
-    ),
-    pytest.mark.filterwarnings(
-        "ignore:"
-        "unclosed transport <_SelectorSocketTransport closing fd=.*"
-        ":ResourceWarning",
-    ),
-]
-
-
-secure_proxy_xfail_under_py310_linux = functools.partial(
-    pytest.mark.xfail,
-    PY_310 and platform.system() == "Linux",
-    reason=(
-        "The secure proxy fixture does not seem to work "
-        "under Python 3.10 on Linux. "
-        "See https://github.com/abhinavsingh/proxy.py/issues/622."
-    ),
-)
-
-ASYNCIO_SUPPORTS_TLS_IN_TLS = hasattr(
-    asyncio.sslproto._SSLProtocolTransport,
-    "_start_tls_compatible",
-)
+ASYNCIO_SUPPORTS_TLS_IN_TLS = sys.version_info >= (3, 11)
 
 
 @pytest.fixture
@@ -51,6 +26,9 @@ def secure_proxy_url(tls_certificate_pem_path):
     This fixture also spawns that instance and tears it down after the test.
     """
     proxypy_args = [
+        # --threadless does not work on windows, see
+        # https://github.com/abhinavsingh/proxy.py/issues/492
+        "--threaded" if os.name == "nt" else "--threadless",
         "--num-workers",
         "1",  # the tests only send one query anyway
         "--hostname",
@@ -112,47 +90,33 @@ async def web_server_endpoint_url(
     )
 
 
-@pytest.fixture
-def _pretend_asyncio_supports_tls_in_tls(
-    monkeypatch,
-    web_server_endpoint_type,
-):
-    if web_server_endpoint_type != "https" or ASYNCIO_SUPPORTS_TLS_IN_TLS:
-        return
-
-    # for https://github.com/python/cpython/pull/28073
-    # and https://bugs.python.org/issue37179
-    monkeypatch.setattr(
-        asyncio.sslproto._SSLProtocolTransport,
-        "_start_tls_compatible",
-        True,
-        raising=False,
-    )
-
-
-@secure_proxy_xfail_under_py310_linux(raises=ClientProxyConnectionError)
+@pytest.mark.skipif(
+    not ASYNCIO_SUPPORTS_TLS_IN_TLS,
+    reason="asyncio on this python does not support TLS in TLS",
+)
 @pytest.mark.parametrize("web_server_endpoint_type", ("http", "https"))
-@pytest.mark.usefixtures("_pretend_asyncio_supports_tls_in_tls", "loop")
+@pytest.mark.filterwarnings(r"ignore:.*ssl.OP_NO_SSL*")
+# Filter out the warning from
+# https://github.com/abhinavsingh/proxy.py/blob/30574fd0414005dfa8792a6e797023e862bdcf43/proxy/common/utils.py#L226
+# otherwise this test will fail because the proxy will die with an error.
 async def test_secure_https_proxy_absolute_path(
-    client_ssl_ctx,
-    secure_proxy_url,
-    web_server_endpoint_url,
-    web_server_endpoint_payload,
+    client_ssl_ctx: ssl.SSLContext,
+    secure_proxy_url: URL,
+    web_server_endpoint_url: str,
+    web_server_endpoint_payload: str,
 ) -> None:
     """Ensure HTTP(S) sites are accessible through a secure proxy."""
     conn = aiohttp.TCPConnector()
     sess = aiohttp.ClientSession(connector=conn)
 
-    response = await sess.get(
+    async with sess.get(
         web_server_endpoint_url,
         proxy=secure_proxy_url,
         ssl=client_ssl_ctx,  # used for both proxy and endpoint connections
-    )
+    ) as response:
+        assert response.status == 200
+        assert await response.text() == web_server_endpoint_payload
 
-    assert response.status == 200
-    assert await response.text() == web_server_endpoint_payload
-
-    response.close()
     await sess.close()
     await conn.close()
 
@@ -160,13 +124,19 @@ async def test_secure_https_proxy_absolute_path(
     await asyncio.sleep(0.1)
 
 
-@secure_proxy_xfail_under_py310_linux(raises=AssertionError)
 @pytest.mark.parametrize("web_server_endpoint_type", ("https",))
 @pytest.mark.usefixtures("loop")
+@pytest.mark.skipif(
+    ASYNCIO_SUPPORTS_TLS_IN_TLS, reason="asyncio on this python supports TLS in TLS"
+)
+@pytest.mark.filterwarnings(r"ignore:.*ssl.OP_NO_SSL*")
+# Filter out the warning from
+# https://github.com/abhinavsingh/proxy.py/blob/30574fd0414005dfa8792a6e797023e862bdcf43/proxy/common/utils.py#L226
+# otherwise this test will fail because the proxy will die with an error.
 async def test_https_proxy_unsupported_tls_in_tls(
-    client_ssl_ctx,
-    secure_proxy_url,
-    web_server_endpoint_type,
+    client_ssl_ctx: ssl.SSLContext,
+    secure_proxy_url: URL,
+    web_server_endpoint_type: str,
 ) -> None:
     """Ensure connecting to TLS endpoints w/ HTTPS proxy needs patching.
 
@@ -208,13 +178,17 @@ async def test_https_proxy_unsupported_tls_in_tls(
         r"$"
     )
 
-    with pytest.warns(RuntimeWarning, match=expected_warning_text,), pytest.raises(
+    with pytest.warns(
+        RuntimeWarning,
+        match=expected_warning_text,
+    ), pytest.raises(
         ClientConnectionError,
         match=expected_exception_reason,
     ) as conn_err:
-        await sess.get(url, proxy=secure_proxy_url, ssl=client_ssl_ctx)
+        async with sess.get(url, proxy=secure_proxy_url, ssl=client_ssl_ctx):
+            pass
 
-    assert type(conn_err.value.__cause__) == TypeError
+    assert isinstance(conn_err.value.__cause__, TypeError)
     assert match_regex(f"^{type_err!s}$", str(conn_err.value.__cause__))
 
     await sess.close()
@@ -272,13 +246,11 @@ def proxy_test_server(aiohttp_raw_server, loop, monkeypatch):
 def get_request(loop):
     async def _request(method="GET", *, url, trust_env=False, **kwargs):
         connector = aiohttp.TCPConnector(ssl=False, loop=loop)
-        client = aiohttp.ClientSession(connector=connector, trust_env=trust_env)
-        try:
-            resp = await client.request(method, url, **kwargs)
-            await resp.release()
-            return resp
-        finally:
-            await client.close()
+        async with aiohttp.ClientSession(
+            connector=connector, trust_env=trust_env
+        ) as client:
+            async with client.request(method, url, **kwargs) as resp:
+                return resp
 
     return _request
 
@@ -418,11 +390,8 @@ async def test_proxy_http_acquired_cleanup_force(proxy_test_server, loop) -> Non
     assert 0 == len(conn._acquired)
 
     async def request():
-        resp = await sess.get(url, proxy=proxy.url)
-
-        assert 1 == len(conn._acquired)
-
-        await resp.release()
+        async with sess.get(url, proxy=proxy.url):
+            assert 1 == len(conn._acquired)
 
     await request()
 
@@ -446,13 +415,11 @@ async def test_proxy_http_multi_conn_limit(proxy_test_server, loop) -> None:
         # process requests only one by one
         nonlocal current_pid
 
-        resp = await sess.get(url, proxy=proxy.url)
+        async with sess.get(url, proxy=proxy.url) as resp:
+            current_pid = pid
+            await asyncio.sleep(0.2, loop=loop)
+            assert current_pid == pid
 
-        current_pid = pid
-        await asyncio.sleep(0.2, loop=loop)
-        assert current_pid == pid
-
-        await resp.release()
         return resp
 
     requests = [request(pid) for pid in range(multi_conn_num)]
@@ -503,9 +470,8 @@ async def xtest_proxy_https_send_body(proxy_test_server, loop):
     proxy.return_value = {"status": 200, "body": b"1" * (2**20)}
     url = "https://www.google.com.ua/search?q=aiohttp proxy"
 
-    resp = await sess.get(url, proxy=proxy.url)
-    body = await resp.read()
-    await resp.release()
+    async with sess.get(url, proxy=proxy.url) as resp:
+        body = await resp.read()
     await sess.close()
 
     assert body == b"1" * (2**20)
@@ -599,11 +565,8 @@ async def xtest_proxy_https_acquired_cleanup(proxy_test_server, loop):
     assert 0 == len(conn._acquired)
 
     async def request():
-        resp = await sess.get(url, proxy=proxy.url)
-
-        assert 1 == len(conn._acquired)
-
-        await resp.release()
+        async with sess.get(url, proxy=proxy.url):
+            assert 1 == len(conn._acquired)
 
     await request()
 
@@ -623,11 +586,8 @@ async def xtest_proxy_https_acquired_cleanup_force(proxy_test_server, loop):
     assert 0 == len(conn._acquired)
 
     async def request():
-        resp = await sess.get(url, proxy=proxy.url)
-
-        assert 1 == len(conn._acquired)
-
-        await resp.release()
+        async with sess.get(url, proxy=proxy.url):
+            assert 1 == len(conn._acquired)
 
     await request()
 
@@ -651,13 +611,11 @@ async def xtest_proxy_https_multi_conn_limit(proxy_test_server, loop):
         # process requests only one by one
         nonlocal current_pid
 
-        resp = await sess.get(url, proxy=proxy.url)
+        async with sess.get(url, proxy=proxy.url) as resp:
+            current_pid = pid
+            await asyncio.sleep(0.2, loop=loop)
+            assert current_pid == pid
 
-        current_pid = pid
-        await asyncio.sleep(0.2, loop=loop)
-        assert current_pid == pid
-
-        await resp.release()
         return resp
 
     requests = [request(pid) for pid in range(multi_conn_num)]
@@ -737,17 +695,17 @@ async def test_proxy_from_env_http_with_auth(proxy_test_server, get_request, moc
 
 
 async def test_proxy_from_env_http_with_auth_from_netrc(
-    proxy_test_server, get_request, tmpdir, mocker
+    proxy_test_server, get_request, tmp_path, mocker
 ):
     url = "http://aiohttp.io/path"
     proxy = await proxy_test_server()
     auth = aiohttp.BasicAuth("user", "pass")
-    netrc_file = tmpdir.join("test_netrc")
+    netrc_file = tmp_path / "test_netrc"
     netrc_file_data = "machine 127.0.0.1 login {} password {}".format(
         auth.login,
         auth.password,
     )
-    with open(str(netrc_file), "w") as f:
+    with netrc_file.open("w") as f:
         f.write(netrc_file_data)
     mocker.patch.dict(
         os.environ, {"http_proxy": str(proxy.url), "NETRC": str(netrc_file)}
@@ -763,17 +721,17 @@ async def test_proxy_from_env_http_with_auth_from_netrc(
 
 
 async def test_proxy_from_env_http_without_auth_from_netrc(
-    proxy_test_server, get_request, tmpdir, mocker
+    proxy_test_server, get_request, tmp_path, mocker
 ):
     url = "http://aiohttp.io/path"
     proxy = await proxy_test_server()
     auth = aiohttp.BasicAuth("user", "pass")
-    netrc_file = tmpdir.join("test_netrc")
+    netrc_file = tmp_path / "test_netrc"
     netrc_file_data = "machine 127.0.0.2 login {} password {}".format(
         auth.login,
         auth.password,
     )
-    with open(str(netrc_file), "w") as f:
+    with netrc_file.open("w") as f:
         f.write(netrc_file_data)
     mocker.patch.dict(
         os.environ, {"http_proxy": str(proxy.url), "NETRC": str(netrc_file)}
@@ -789,14 +747,14 @@ async def test_proxy_from_env_http_without_auth_from_netrc(
 
 
 async def test_proxy_from_env_http_without_auth_from_wrong_netrc(
-    proxy_test_server, get_request, tmpdir, mocker
+    proxy_test_server, get_request, tmp_path, mocker
 ):
     url = "http://aiohttp.io/path"
     proxy = await proxy_test_server()
     auth = aiohttp.BasicAuth("user", "pass")
-    netrc_file = tmpdir.join("test_netrc")
+    netrc_file = tmp_path / "test_netrc"
     invalid_data = f"machine 127.0.0.1 {auth.login} pass {auth.password}"
-    with open(str(netrc_file), "w") as f:
+    with netrc_file.open("w") as f:
         f.write(invalid_data)
 
     mocker.patch.dict(
@@ -863,8 +821,9 @@ async def test_proxy_auth() -> None:
         with pytest.raises(
             ValueError, match=r"proxy_auth must be None or BasicAuth\(\) tuple"
         ):
-            await session.get(
+            async with session.get(
                 "http://python.org",
                 proxy="http://proxy.example.com",
                 proxy_auth=("user", "pass"),
-            )
+            ):
+                pass
